'' =================================================================================================
''
''   File....... jm_lanc_demo.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2014 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 05 APR 2014
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28

  SIO = 26                                                      ' lanc serial io
  

con { pst formatting }

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR
  #14, GOTOX, GOTOY, CLS


obj

  lanc : "jm_lanc"
  term : "fullduplexserial"
 

var


pub main | f1, f2, c, p_buf, check

  lanc.start(SIO)                                               ' lanc rx
  term.start(RX1, TX1, %0000, 115_200)                          ' terminal
  pause(5)

  term.rxflush
  term.tx(CLS)

  f1 := lanc.frame
  
  repeat
    c := ucase(term.rxcheck)                                    ' check keyboard
    if (c => 0)                                                 ' if key...
      case c
        "P": { power }
          if (lanc.available)                                   ' lanc signal present?
            lanc.command($18, $5E)                              ' power down
          else
            lanc.wake_up                                        ' attempt wake-up

        "R": { toggle record }
          lanc.command(18, $33) 
          
        "A": { toggle auto focus }
          lanc.command($28, $41)     

        "N": { focus near }
          lanc.command($28, $47) 

        "F": { focus far }
          lanc.command($28, $45)

        "T": { zoom tele - fast }
          lanc.command($28, $0E) 
           
        "W": { zoom wide - fast }   
          lanc.command($28, $1E)
          
    f2 := lanc.frame
    
    if (f2 == 0)
      term.str(string("No LANC signal", CR))
      pause(16)
      
    elseif (f2 <> f1)                                           ' new frame?
      p_buf := lanc.address                                     ' display lanc bytes
      repeat 8                                         
        term.hex(byte[p_buf++], 2)
        term.tx(" ")

      check := byte[lanc.address + 4]                           ' status
      case check
        $02:   term.str(string(" VTR - STOP   "))
        $03:   term.str(string(" VTR - FWD    "))
        $06:   term.str(string(" VTR - PLAY   ")) 
        $07:   term.str(string(" VTR - PAUSE  "))
        $46:   term.str(string(" VTR - PLAY >>"))
        $56:   term.str(string(" VTR - PLAY <<"))   
        $83:   term.str(string(" VTR - RWD    "))

        $14:   term.str(string(" CAM - PAUSE  "))   
        $04:   term.str(string(" CAM - RECORD "))

      term.tx(CR)
      
    f1 := f2


pub ucase(c)

  if ((c => "a") and (c =< "z"))
    c -= 32

  return c


con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]


dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  